#ifndef MPEG_PLAY_H
#define MPEG_PLAY_H

#include <X11/Xlib.h>  /* Display, Window */

typedef enum {
    ORDERED2_DITHER,
    HYBRID_DITHER,
    FS2FAST_DITHER,
    Twox2_DITHER,
    FULL_COLOR_DITHER,
    GRAY_DITHER,
    GRAY2_DITHER
} DitherType;

#ifndef MPEG_STREAM_DEFINED
#define MPEG_STREAM_DEFINED
struct MpegStreamContainer;
typedef struct MpegStreamContainer *MpegStream;
#endif

/* Return values from functions. */
/* remember to update mpRetStr in mpeg_api.c when changing this one! */
enum {
    MPEG_ERR_CHILD = -128,  /* error communicating with child. */
    MPEG_ERR_UNKNOWN,       /* child received unknown command. */
    MPEG_ERR_OVERFLOW,      /* command buffer overflow */
    MPEG_ERR_NO_MOVIE,      /* no movie present. */
    MPEG_ERR_NOT_SEEKABLE,  /* stream is not seekable. */
    MPEG_OK = 0,            /* success. */
    MPEG_FILM_HAS_ENDED     /* end of film reached. */
};

/* mapping return values to strings */
char *mpRetStr(int n);

/* video stream creation */
MpegStream mpNewStreamFromFile(const char *filename,
			       Display *display,
			       Window parent,
			       Window applicationMainWindow,
			       DitherType dither,
			       int frameRate,
			       int loop,
			       double gammaCorrection,
			       double chromaCorrection,
			       int useSharedMemory,
			       int useOwnColorMap);
MpegStream mpNewStreamPushed(Display *display,
			     Window parent,
			     Window applicationMainWindow,
			     DitherType dither,
			     int frameRate,
			     double gammaCorrection,
			     double chromaCorrection,
			     int useSharedMemory,
			     int useOwnColorMap);

/* video stream control */
int mpPlay(MpegStream stream);
int mpNextFrame(MpegStream stream);
int mpRewind(MpegStream stream);
int mpStop(MpegStream stream);
int mpQuit(MpegStream stream);
int mpSetLoop(MpegStream stream, int onoff);

/* window handling */
int mpRepaint(MpegStream stream);
int mpParentResize(MpegStream stream);

/* for Pushed streams */
int mpQueryWantedBytes(MpegStream stream, int *n);
int mpSendBytes(MpegStream stream, void *buffer, int *n);
int mpEndOfStream(MpegStream stream);

#endif
