#ifndef MPEG_PROC_H
#define MPEG_PROC_H

#include <X11/Xlib.h>  /* Display, Window */
#include <sys/types.h> /* pid_t */
#include <sys/time.h>  /* struct timeval */

#include "mpeg_play.h" /* DitherType */

struct MpegStreamContainer {
    int pipeCmd;       /* write end of command pipe. */
    int pipeStat;      /* read end of status pipe. */
    pid_t child;       /* pid of video decoding process. */
};
#ifndef MPEG_STREAM_DEFINED
#define MPEG_STREAM_DEFINED
typedef struct MpegStreamContainer *MpegStream;
#endif

enum {
    MPEG_CMD_PLAY,
    MPEG_CMD_NEXT_FRAME,
    MPEG_CMD_REWIND,
    MPEG_CMD_PAUSE,
    MPEG_CMD_QUIT,
    MPEG_CMD_REPAINT,
    MPEG_CMD_PARENT_RESIZE,
    MPEG_CMD_SET_LOOP,
    MPEG_CMD_QUERY_WANTED_BYTES,
    MPEG_CMD_SEND_BYTES,
    MPEG_CMD_END_OF_STREAM
};

extern int mpThisIsTheChild;

int  mpFetchCommandOrData(struct timeval *tv, int waitForData);
void mpDoMove(unsigned w, unsigned h);

void mpClientReturnStatus(int status);
int  mpServerReceiveStatus(MpegStream stream);
void mpClientReturnInt(int status);
int  mpServerReceiveInt(MpegStream stream);
int  mpServerSendCommand(MpegStream stream, int command);
int  mpServerSendIntCommand(MpegStream stream, int command, int parameter);
int  mpClientReceiveCommand(void);
int  mpClientReceiveIntParameter(void);

MpegStream mpNewStream(Display **dpy);

void mpChildNewStreamFromFile(const char *filename,
			      Display *display,
			      Window parent,
			      Window applicationMainWindow,
			      DitherType dither,
			      int frameRate,
			      int loop,
			      double gammaCorrection,
			      double chromaCorrection,
			      int useSharedMemory,
			      int useOwnColorMap);
void mpChildNewStreamPushed(Display *display,
			    Window parent,
			    Window applicationMainWindow,
			    DitherType dither,
			    int frameRate,
			    double gammaCorrection,
			    double chromaCorrection,
			    int useSharedMemory,
			    int useOwnColorMap);

#endif
