/* this file is CLEAN2 [sverrehu] */

/* Miscellaneous functions that deal with >= 24 bit color displays. */

/* 
 * Copyright (c) 1995 The Regents of the University of California.
 * All rights reserved.
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice and the following
 * two paragraphs appear in all copies of this software.
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF
 * CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

/* 
 * Portions of this software Copyright (c) 1995 Brown University.
 * All rights reserved.
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement
 * is hereby granted, provided that the above copyright notice and the
 * following two paragraphs appear in all copies of this software.
 * 
 * IN NO EVENT SHALL BROWN UNIVERSITY BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF BROWN
 * UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * BROWN UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS"
 * BASIS, AND BROWN UNIVERSITY HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include "video.h"
#include "proto.h"

/* 
 *--------------------------------------------------------------
 *
 * FindFullColorVisual
 *
 *  Returns a pointer to a full color bit visual on the display
 *
 * Results:
 *      See above.
 *  
 * Side effects:
 *      Unknown.
 *
 *--------------------------------------------------------------
 */
Visual *FindFullColorVisual(Display *dpy, int *depth)
{
    XVisualInfo vinfo;
    XVisualInfo *vinfo_ret;
    int numitems, maxdepth;

    vinfo.class = TrueColor;
    vinfo_ret = XGetVisualInfo(dpy, VisualClassMask, &vinfo, &numitems);
    if (numitems == 0)
	return NULL;
    maxdepth = 0;
    while (numitems > 0) {
	if (vinfo_ret[numitems - 1].depth > maxdepth)
	    maxdepth = vinfo_ret[numitems - 1].depth;
	numitems--;
    }
    XFree((void *) vinfo_ret);
    if (maxdepth < 16)
	return NULL;
    if (XMatchVisualInfo(dpy, DefaultScreen(dpy), maxdepth,
			 TrueColor, &vinfo)) {
	*depth = maxdepth;
	return vinfo.visual;
    }
    return NULL;
}

/* 
 *--------------------------------------------------------------
 *
 * CreateFullColorWindow
 *
 *  Creates a window capable of handling 32 bit color.
 *
 * Results:
 *      See above.
 *  
 * Side effects:
 *      Unknown.
 *
 *--------------------------------------------------------------
 */
void CreateFullColorWindow(XInfo *xinfo)
{
    Display *display = xinfo->display;
    int screen;
    Visual *vis;
    XSetWindowAttributes attrib;
    unsigned long attrib_flags = 0;
    int depth;

    screen = XDefaultScreen(display);
    if (xinfo->visual == NULL) {
	xinfo->visual = vis = FindFullColorVisual(display, &depth);
	xinfo->depth = depth;
    } else {
	vis = xinfo->visual;
	depth = xinfo->depth;
    }
    if (vis == NULL)
	return;
    if (xinfo->cmap == 0)
	attrib.colormap = XCreateColormap(display, xinfo->parent,
					  vis, AllocNone);
    else
	attrib.colormap = xinfo->cmap;
    attrib_flags |= CWColormap;
    attrib.background_pixel = WhitePixel(display, screen);
    attrib.border_pixel = BlackPixel(display, screen);
    attrib.backing_store = NotUseful;
    attrib.save_under = False;
    attrib_flags |= CWBackPixel | CWBorderPixel | CWBackingStore | CWSaveUnder;
    xinfo->window = XCreateWindow(display, xinfo->parent,
				  xinfo->hints.x, xinfo->hints.y,
				  xinfo->hints.width, xinfo->hints.height,
				  0, xinfo->depth, InputOutput, vis,
				  attrib_flags, &attrib);
}
