/* $Id$ */
/**************************************************************************
 *
 *  FILE            xfile.c
 *  MODULE OF       mpeg_play library.
 *
 *  DESCRIPTION     
 *
 *  WRITTEN BY      Sverre H. Huseby <sverrehu@ifi.uio.no>
 *
 **************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <setjmp.h>
#include <string.h>
#include <unistd.h>

#include "mpeg_proc.h"
#include "xfile.h"

#ifndef min
#define min(a,b) ((a) < (b) ? (a) : (b))
#endif

/**************************************************************************
 *                                                                        *
 *                   P R I V A T E    F U N C T I O N S                   *
 *                                                                        *
 **************************************************************************/

static int
mpfNeedMoreData(void)
{
    return mpFetchCommandOrData(NULL, 1);
}



/**************************************************************************
 *                                                                        *
 *                    P U B L I C    F U N C T I O N S                    *
 *                                                                        *
 **************************************************************************/

XFILE *
mpfNewXFile(int bufferSize)
{
    XFILE *xf;

    if ((xf = malloc(sizeof(XFILE))) == NULL)
	return NULL;
    if ((xf->buf = malloc(bufferSize)) == NULL) {
	free(xf);
	return NULL;
    }
    xf->len = bufferSize;
    xf->in = xf->out = xf->n = 0;
    return xf;
}

int
mpfQueryWantedBytes(XFILE *xf)
{
    return xf->len - xf->n;
}

int
mpfGetDataFromFD(XFILE *xf, int fd, int n)
{
    int bytesRead = 0, toRead, q;

    if (n > mpfQueryWantedBytes(xf))
	return -1;
    toRead = xf->len - xf->in;
    if (toRead > n)
	toRead = n;
    while (toRead) {
	if ((q = read(fd, xf->buf + xf->in, toRead)) < 0)
	    return -1;
	xf->in += q;
	xf->n += q;
	toRead -= q;
	bytesRead += q;
    }
    if (xf->in == xf->len)
	xf->in = 0;
    if (bytesRead < n) {
	toRead = n - bytesRead;
	while (toRead) {
	    if ((q = read(fd, xf->buf + xf->in, toRead)) < 0)
		return -1;
	    xf->in += q;
	    xf->n += q;
	    toRead -= q;
	    bytesRead += q;
	}
    }
    if (xf->in == xf->len)
	xf->in = 0;
    return bytesRead;
}

int
mpfRead(XFILE *xf, void *buf, int n)
{
    int ret, toRead, len1, len2;

    toRead = n;
    while (toRead) {
	if (!xf->n)
	    if ((ret = mpfNeedMoreData()) < 0)
		return ret;
	    else if (ret == 0)
		return n - toRead;
	len1 = min(xf->n, toRead);
	if (len1 <= (len2 = xf->len - xf->out)) {
	    memcpy(buf, xf->buf + xf->out, len1);
	    if ((xf->out += len1) >= xf->len)
		xf->out = 0;
	    (unsigned char *) buf += len1;
	} else {
	    memcpy(buf, xf->buf + xf->out, len2);
	    xf->out = 0;
	    (unsigned char *) buf += len2;
	    len2 = len1 - len2;
	    memcpy(buf, xf->buf + xf->out, len2);
	    if ((xf->out += len2) >= xf->len)
		xf->out = 0;
	    (unsigned char *) buf += len2;
	}
	toRead -= len1;
	xf->n -= len1;
    }
    return n;
}

int
mpfGetc(XFILE *xf)
{
    int ret;

    if (!xf->n)
	if (mpfNeedMoreData() <= 0)
	    return EOF;
    ret = xf->buf[xf->out];
    if (++xf->out == xf->len)
	xf->out = 0;
    --xf->n;
    return ret;
}

int
mpfSkip(XFILE *xf, int n)
{
    while (n) {
	if (n <= xf->n) {
	    xf->n -= n;
	    if ((xf->out += n) >= xf->len)
		xf->out -= xf->len;
	    n = 0;
	} else {
	    if ((xf->out += xf->n) >= xf->len)
		xf->out -= xf->len;
	    n -= xf->n;
	    xf->n = 0;
	    if (!xf->n)
		if (mpfNeedMoreData() <= 0)
		    return -1;
	}
    }
    return 0;
}
