/* $Id$ */
/**************************************************************************
 *
 *  FILE            xselect.c
 *  MODULE OF       mpeg_play library.
 *
 *  DESCRIPTION     
 *
 *  WRITTEN BY      Sverre H. Huseby
 *
 *  MODIFICATIONS   
 *
 **************************************************************************/

#include <stdlib.h>
#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include <errno.h>

#include "xselect.h"

/**************************************************************************
 *                                                                        *
 *                    P U B L I C    F U N C T I O N S                    *
 *                                                                        *
 **************************************************************************/

/*-------------------------------------------------------------------------
 *
 *  NAME          xselect
 *
 *  FUNCTION      select(2)-wrapper.
 *
 *  DESCRIPTION   This works like the normal select, except from the
 *                following:
 *                  * it will correctly handle EINTR-errors by checking how
 *                    much of the timeout is left, instead of starting all
 *                    over again.
 *                  * when a file descriptor status change is returned, the
 *                    timeout value is updated to contain the time left
 *                    to wait for a full timeout.
 */
int
xselect(int n, fd_set *readfds, fd_set *writefds, fd_set *exceptfds,
	struct timeval *timeout)
{
    int    ret;
    struct timeval tv, before, after, *tvp;

    do {
	if (timeout) {
	    gettimeofday(&before, NULL);
	    tv = *timeout;
	    tvp = &tv;
	} else
	    tvp = NULL;
        ret = select(n, readfds, writefds, exceptfds, tvp);
	if (timeout) {
	    if (ret == 0) {
		timeout->tv_sec = timeout->tv_usec = 0L;
	    } else {
		gettimeofday(&after, NULL);
		after.tv_sec -= before.tv_sec;
		if ((after.tv_usec -= before.tv_usec) < 0L) {
		    after.tv_usec += 1000000L;
		    --after.tv_sec;
		}
		timeout->tv_sec -= after.tv_sec;
		if ((timeout->tv_usec -= after.tv_usec) < 0L) {
		    timeout->tv_usec += 1000000L;
		    if (--timeout->tv_sec < 0L)
			timeout->tv_sec = timeout->tv_usec = 0L;
		}
	    }
	}
    } while (ret < 0 && errno == EINTR);
    return ret;
}
