/* $Id$ */
/**************************************************************************
 *
 *  FILE            rcfile.c
 *  MODULE OF       SHH MPEG plugin for Netscape
 *
 *  DESCRIPTION     Parse the startup file for this plugin.
 *
 *  WRITTEN BY      Sverre H. Huseby <sverrehu@ifi.uio.no>
 *
 **************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "parsefile.h"
#include "shhmpeg.h"
#include "rcfile.h"

/**************************************************************************
 *                                                                        *
 *                   P R I V A T E    F U N C T I O N S                   *
 *                                                                        *
 **************************************************************************/

static int
rcParseError(const char *s)
{
    fprintf(stderr,
	    "%s in `%s', line %d\n",
	     s, pfGetCurrentFile(), pfGetCurrentLine());
    return -1;
}

static int
rcCheckAssignment(int argc, char *argv[])
{
    if (argc == 1 || strcmp(argv[1], "=") != 0)
	return rcParseError("missing `='");
    else if (argc == 2 || !strlen(argv[2]))
	return rcParseError("missing argument to assignment");
    else if (argc > 3)
	return rcParseError("too many arguments to assignment");
    return 0;
}

static int
rcCheckBooleanAssignment(int argc, char *argv[])
{
    if (rcCheckAssignment(argc, argv) != 0 || pfGetBooleanValue(argv[2]) == -1)
	return -1;
    return 0;
}

static int
rcParseLine(int argc, char *argv[], void *data)
{
    PluginInstance *This = (PluginInstance *) data;

    if (pfStrCaseCmp(argv[0], "DefaultTransferMode") == 0) {
	if (rcCheckAssignment(argc, argv) == 0) {
	    if (pfStrCaseCmp(argv[2], "Stream") == 0)
		This->opmode = NP_NORMAL;
	    else if (pfStrCaseCmp(argv[2], "LocalFile") == 0)
		This->opmode = NP_ASFILEONLY;
	    else
		rcParseError("invalid argument to DefaultTransferMode");
	}
    } else if (pfStrCaseCmp(argv[0], "DefaultAutoPlay") == 0) {
	if (rcCheckBooleanAssignment(argc, argv) == 0)
	    This->autoStart = pfGetBooleanValue(argv[2]);
    } else if (pfStrCaseCmp(argv[0], "DefaultLoop") == 0) {
	if (rcCheckBooleanAssignment(argc, argv) == 0)
	    This->doLoop = pfGetBooleanValue(argv[2]);
    } else if (pfStrCaseCmp(argv[0], "DefaultFrameRate") == 0) {
	if (rcCheckAssignment(argc, argv) == 0)
	    This->frameRate = atoi(argv[2]);
    } else if (pfStrCaseCmp(argv[0], "DefaultDoubleSize") == 0) {
	if (rcCheckBooleanAssignment(argc, argv) == 0)
	    This->doubleSize = pfGetBooleanValue(argv[2]);
    } else if (pfStrCaseCmp(argv[0], "UseSharedMemory") == 0) {
	if (rcCheckBooleanAssignment(argc, argv) == 0)
	    This->useSharedMemory = pfGetBooleanValue(argv[2]);
    } else if (pfStrCaseCmp(argv[0], "UseOwnColormap") == 0) {
	if (rcCheckBooleanAssignment(argc, argv) == 0)
	    This->useOwnColormap = pfGetBooleanValue(argv[2]);
    } else if (pfStrCaseCmp(argv[0], "Dither") == 0) {
	if (rcCheckAssignment(argc, argv) == 0) {
	    if (pfStrCaseCmp(argv[2], "Ordered") == 0)
		This->dither = ORDERED2_DITHER;
	    else if (pfStrCaseCmp(argv[2], "Hybrid") == 0)
		This->dither = HYBRID_DITHER;
	    else if (pfStrCaseCmp(argv[2], "FsFast") == 0)
		This->dither = FS2FAST_DITHER;
	    else if (pfStrCaseCmp(argv[2], "FullColor") == 0)
		This->dither = FULL_COLOR_DITHER;
	    else if (pfStrCaseCmp(argv[2], "Gray") == 0)
		This->dither = GRAY_DITHER;
	    else
		rcParseError("invalid argument to Dither");
	}
    } else {
	rcParseError("syntax error");
    }
    return 1; /* continue parsing */
}


/**************************************************************************
 *                                                                        *
 *                    P U B L I C    F U N C T I O N S                    *
 *                                                                        *
 **************************************************************************/

int
rcRead(PluginInstance *This)
{
    int  ret;
    char *home, *filename;

    /* read rc-file in user's home directory */
    if ((home = getenv("HOME")) != NULL) {
	filename = malloc(strlen(home) + strlen(RC_FILE) + 2);
	strcpy(filename, home);
	strcat(filename, "/");
	strcat(filename, RC_FILE);
	if ((ret = pfParseFile(filename, rcParseLine, 1, This)) != 0) {
	    free(filename);
	    return ret;
	}
	free(filename);
    }
    /* read rc-file in current working directory */
    ret = pfParseFile(RC_FILE, rcParseLine, 1, This);

    return ret;
}
