/* $Id$ */
/**************************************************************************
 *
 *  FILE            shhmpeg.c
 *  MODULE OF       SHH MPEG plugin for Netscape
 *
 *  DESCRIPTION     
 *
 *  WRITTEN BY      Sverre H. Huseby <sverrehu@ifi.uio.no>
 *
 **************************************************************************/

#include <stdio.h>
#include "shhmpeg.h"
#include "parsefile.h"
#include "rcfile.h"

#include <unistd.h>
#include <Xm/Form.h>
#include <Xm/PushB.h>
#include <Xm/ToggleB.h>
#include <Xm/DrawingA.h>
#include <Xm/Label.h>
#include <mpeg_play.h>

#define FONT_NAME "6x10"
#define STREAM_BUF_SIZE 512

/**************************************************************************
 *                                                                        *
 *                   P R I V A T E    F U N C T I O N S                   *
 *                                                                        *
 **************************************************************************/

#ifdef DIRTY_FIND_TOP_LEVEL_WINDOW
static Window
xGetParentWindow(Display *dpy, Window w)
{
    Window root, parent, *children;
    unsigned nchildren;

    if (XQueryTree(dpy, w, &root, &parent, &children, &nchildren) != 0) {
	XFree(children);
	return parent;
    }
    return 0;
}

static Window
findTopLevelWindow(Display *dpy, Window w)
{
    XWMHints *hints;

    do {
	if ((hints = XGetWMHints(dpy, w)) != NULL) {
	    XFree(hints);
	    return w;
	} else
        w = xGetParentWindow(dpy, w);
    } while (w);
    return w;
}
#else
static Window
findTopLevelWindow(Display *dpy, Window w)
{
    Widget wdg;

    wdg = XtWindowToWidget(dpy, w);
    while(wdg && !XtIsTopLevelShell(wdg))
	wdg = XtParent(wdg);
    return XtWindowOfObject(wdg);
}
#endif

static void
ButtCallback(Widget w, XtPointer clientData, XtPointer callData)
{
    PluginInstance *This = (PluginInstance *) clientData;
    int ret = MPEG_OK;

    if (w == This->buttRewind)
	ret = mpRewind(This->mpegStream);
    else if (w == This->buttPlay)
	ret = mpPlay(This->mpegStream);
    else if (w == This->buttPause)
	ret = mpStop(This->mpegStream);
    else if (w == This->buttStep)
	ret = mpNextFrame(This->mpegStream);
    else if (w == This->buttLoop)
	ret = mpSetLoop(This->mpegStream, This->doLoop ^= 1);
    if (ret != MPEG_OK)
	fprintf(stderr, "%s\n", mpRetStr(ret));
}

static void
VideoExposeCallback(Widget w, XtPointer clientData, XtPointer callData)
{
    PluginInstance *This = (PluginInstance *) clientData;

    XClearWindow(This->display, XtWindowOfObject(This->video));
    XFlush(This->display);
    mpRepaint(This->mpegStream);
}



/**************************************************************************
 *                                                                        *
 *                    P U B L I C    F U N C T I O N S                    *
 *                                                                        *
 **************************************************************************/

char *
NPP_GetMIMEDescription(void)
{
    return ("video/mpeg:mpeg,mpg,mpe:MPEG Video");
}

NPError
NPP_GetValue(void *future, NPPVariable variable, void *value)
{
    NPError err = NPERR_NO_ERROR;

    switch (variable) {
      case NPPVpluginNameString:
	*((char **) value) = "MPEG video stream plugin";
	break;
      case NPPVpluginDescriptionString:
	*((char **) value) =
	    "Inline display of MPEG video streams (no sound).";
	break;
      default:
	err = NPERR_GENERIC_ERROR;
    }
    return err;
}

NPError
NPP_Initialize(void)
{
    return NPERR_NO_ERROR;
}

jref
NPP_GetJavaClass(void)
{
    return NULL;
}

void
NPP_Shutdown(void)
{
}

NPError
NPP_New(NPMIMEType pluginType, NPP instance, uint16 mode, int16 argc,
	char *argn[], char *argv[], NPSavedData *saved)
{
    PluginInstance *This;
    int q, bool;

    if (instance == NULL)
	return NPERR_INVALID_INSTANCE_ERROR;

    instance->pdata = NPN_MemAlloc(sizeof(PluginInstance));
    if ((This = (PluginInstance *) instance->pdata) == NULL)
	return NPERR_OUT_OF_MEMORY_ERROR;

    /* mode is NP_EMBED, NP_FULL, or NP_BACKGROUND (see npapi.h) */
    This->mode = mode;
    This->window = (Window) 0;
    This->form = 0;
    This->opmode = NP_NORMAL /* NP_ASFILEONLY */;
    This->buttRewind = This->buttPlay = This->buttPause = 0;
    This->buttStep = This->buttLoop = This->video = This->brag = 0;
    This->mpegStream = NULL;
    This->doLoop = 1;
    This->frameRate = 0;
    This->autoStart = 0;
    This->doubleSize = 0;
    This->useSharedMemory = 1;
    This->useOwnColormap = 1;
    This->dither = ORDERED2_DITHER;
    This->pushed = 0;

    rcRead(This);

    for (q = 0; q < argc; q++) {
	if (pfStrCaseCmp(argn[q], "TransferMode") == 0 && argv[q]) {
	    if (pfStrCaseCmp(argv[q], "LocalFile") == 0)
		This->opmode = NP_ASFILEONLY;
	    else if (pfStrCaseCmp(argv[q], "Stream") == 0)
		This->opmode = NP_NORMAL;
	} else if (pfStrCaseCmp(argn[q], "AutoPlay") == 0 && argv[q]) {
	    if ((bool = pfGetBooleanValue(argv[q])) != -1)
		This->autoStart = bool;
	} else if (pfStrCaseCmp(argn[q], "FrameRate") == 0 && argv[q]) {
	    This->frameRate = atoi(argv[q]);
	} else if (pfStrCaseCmp(argn[q], "Loop") == 0 && argv[q]) {
	    if ((bool = pfGetBooleanValue(argv[q])) != -1)
		This->doLoop = bool;
	} else if (pfStrCaseCmp(argn[q], "DoubleSize") == 0 && argv[q]) {
	    if ((bool = pfGetBooleanValue(argv[q])) != -1)
		This->doubleSize = bool;
	}
    }

    if (This->doubleSize) {
	if (This->dither == GRAY_DITHER)
	    This->dither = GRAY2_DITHER;
	else
	    This->dither = Twox2_DITHER;
    }

    /* PLUGIN DEVELOPERS: Initialize fields of your plugin
     * instance data here. If the NPSavedData is non-NULL, you
     * can use that data (returned by you from NPP_Destroy to set
     * up the new plugin instance). */

    return NPERR_NO_ERROR;
}

NPError
NPP_Destroy(NPP instance, NPSavedData **save)
{
    PluginInstance *This;

    if (instance == NULL)
	return NPERR_INVALID_INSTANCE_ERROR;

    This = (PluginInstance *) instance->pdata;

    /* PLUGIN DEVELOPERS: If desired, call NP_MemAlloc to create a
     * NPSavedDate structure containing any state information that you
     * want restored if this plugin instance is later recreated. */

    if (This != NULL) {
	mpQuit(This->mpegStream);
	NPN_MemFree(instance->pdata);
	instance->pdata = NULL;
    }
    return NPERR_NO_ERROR;
}

NPError
NPP_SetWindow(NPP instance, NPWindow *window)
{
    PluginInstance *This;

    if (instance == NULL)
	return NPERR_INVALID_INSTANCE_ERROR;

    if (window == NULL)
	return NPERR_NO_ERROR;

    This = (PluginInstance *) instance->pdata;
    This->x = window->x;
    This->y = window->y;
    This->width = window->width;
    This->height = window->height;

    /* PLUGIN DEVELOPERS: Before setting window to point to the new
     * window, you may wish to compare the new window info to the
     * previous window (if any) to note window size changes, etc. */

    if (This->window == (Window) window->window) {
	/* This is a plugin changing size */
	XtVaSetValues(This->form, XmNx, This->x, XmNy, This->y,
		      XmNheight, This->height, XmNwidth, This->width, 0);
	XFlush(This->display);
	mpParentResize(This->mpegStream);
    } else {
	This->window = (Window) window->window;
	This->display = ((NPSetWindowCallbackStruct *)
			 window->ws_info)->display;
	{
	    Widget netscape_widget;
	    Arg av[20];
	    int ac;
	    XmFontListEntry xmFontEntry;
	    XmFontList xmFont;

	    netscape_widget = XtWindowToWidget(This->display, This->window);
	    if ((xmFontEntry
		 = XmFontListEntryLoad(This->display, FONT_NAME,
				       XmFONT_IS_FONT,
				       XmFONTLIST_DEFAULT_TAG)) != NULL) {
		xmFont = XmFontListAppendEntry(NULL, xmFontEntry);
		if (xmFont == NULL)
		    fprintf(stderr, "unable to create fontlist\n");
	    } else {
		xmFont = NULL;
		fprintf(stderr, "unable to load font `%s'\n", FONT_NAME);
	    }

	    /*
	     *  Argh! XtSetArg is a _macro_, and doesn't like operands
	     *  with side effects.
	     */
	    ac = 0;
	    XtSetArg(av[ac], XmNx, 0);
	    ++ac;
	    XtSetArg(av[ac], XmNy, 0);
	    ++ac;
	    XtSetArg(av[ac], XmNwidth, This->width);
	    ++ac;
	    XtSetArg(av[ac], XmNheight, This->height);
	    ++ac;
	    XtSetArg(av[ac], XmNborderWidth, 0);
	    ++ac;
	    This->form
		= XmCreateForm(netscape_widget, "pluginForm", av, ac);

	    ac = 0;
	    XtSetArg(av[ac], XmNbottomAttachment, XmATTACH_FORM);
	    ++ac;
	    XtSetArg(av[ac], XmNleftAttachment, XmATTACH_FORM);
	    ++ac;
	    if (xmFont) {
		XtSetArg(av[ac], XmNfontList, xmFont);
		++ac;
	    }
	    This->buttRewind
		= XmCreatePushButton(This->form, "Rewind", av, ac);
	    XtAddCallback(This->buttRewind, XmNactivateCallback,
			  ButtCallback, This);

	    ac = 0;
	    XtSetArg(av[ac], XmNbottomAttachment, XmATTACH_FORM);
	    ++ac;
	    XtSetArg(av[ac], XmNleftWidget, This->buttRewind);
	    ++ac;
	    XtSetArg(av[ac], XmNleftAttachment, XmATTACH_WIDGET);
	    ++ac;
	    if (xmFont) {
		XtSetArg(av[ac], XmNfontList, xmFont);
		++ac;
	    }
	    This->buttPlay
		= XmCreatePushButton(This->form, "Play", av, ac);
	    XtAddCallback(This->buttPlay, XmNactivateCallback,
			  ButtCallback, This);

	    ac = 0;
	    XtSetArg(av[ac], XmNbottomAttachment, XmATTACH_FORM);
	    ++ac;
	    XtSetArg(av[ac], XmNleftWidget, This->buttPlay);
	    ++ac;
	    XtSetArg(av[ac], XmNleftAttachment, XmATTACH_WIDGET);
	    ++ac;
	    if (xmFont) {
		XtSetArg(av[ac], XmNfontList, xmFont);
		++ac;
	    }
	    This->buttPause
		= XmCreatePushButton(This->form, "Pause", av, ac);
	    XtAddCallback(This->buttPause, XmNactivateCallback,
			  ButtCallback, This);

	    ac = 0;
	    XtSetArg(av[ac], XmNbottomAttachment, XmATTACH_FORM);
	    ++ac;
	    XtSetArg(av[ac], XmNleftWidget, This->buttPause);
	    ++ac;
	    XtSetArg(av[ac], XmNleftAttachment, XmATTACH_WIDGET);
	    ++ac;
	    if (xmFont) {
		XtSetArg(av[ac], XmNfontList, xmFont);
		++ac;
	    }
	    This->buttStep
		= XmCreatePushButton(This->form, "Step", av, ac);
	    XtAddCallback(This->buttStep, XmNactivateCallback,
			  ButtCallback, This);

	    ac = 0;
	    XtSetArg(av[ac], XmNbottomAttachment, XmATTACH_FORM);
	    ++ac;
	    XtSetArg(av[ac], XmNrightAttachment, XmATTACH_FORM);
	    ++ac;
	    XtSetArg(av[ac], XmNset, This->doLoop);
	    ++ac;
	    if (xmFont) {
		XtSetArg(av[ac], XmNfontList, xmFont);
		++ac;
	    }
	    This->buttLoop
		= XmCreateToggleButton(This->form, "Loop", av, ac);
	    XtAddCallback(This->buttLoop, XmNvalueChangedCallback,
			  ButtCallback, This);

	    ac = 0;
	    XtSetArg(av[ac], XmNtopAttachment, XmATTACH_FORM);
	    ++ac;
	    XtSetArg(av[ac], XmNleftAttachment, XmATTACH_FORM);
	    ++ac;
	    XtSetArg(av[ac], XmNrightAttachment, XmATTACH_FORM);
	    ++ac;
	    XtSetArg(av[ac], XmNbottomWidget, This->buttRewind);
	    ++ac;
	    XtSetArg(av[ac], XmNbottomAttachment, XmATTACH_WIDGET);
	    ++ac;
	    This->video = XmCreateDrawingArea(This->form, "Video", av, ac);
	    XtAddCallback(This->video, XmNexposeCallback,
			  VideoExposeCallback, This);

	    ac = 0;
	    XtSetArg(av[ac], XmNbottomAttachment, XmATTACH_FORM);
	    ++ac;
	    XtSetArg(av[ac], XmNtopAttachment, XmATTACH_WIDGET);
	    ++ac;
	    XtSetArg(av[ac], XmNtopWidget, This->video);
	    ++ac;
	    XtSetArg(av[ac], XmNleftWidget, This->buttStep);
	    ++ac;
	    XtSetArg(av[ac], XmNleftAttachment, XmATTACH_WIDGET);
	    ++ac;
	    XtSetArg(av[ac], XmNrightWidget, This->buttLoop);
	    ++ac;
	    XtSetArg(av[ac], XmNrightAttachment, XmATTACH_WIDGET);
	    ++ac;
	    XtSetArg(av[ac], XmNalignment, XmALIGNMENT_CENTER);
	    ++ac;
	    if (xmFont) {
		XtSetArg(av[ac], XmNfontList, xmFont);
		++ac;
	    }
	    This->brag
		= XmCreateLabel(This->form, "SHH MPEG Plugin", av, ac);

	    XtManageChild(This->buttRewind);
	    XtManageChild(This->buttPlay);
	    XtManageChild(This->buttPause);
	    XtManageChild(This->buttStep);
	    XtManageChild(This->buttLoop);
	    XtManageChild(This->video);
	    XtManageChild(This->brag);
	    XtManageChild(This->form);
	}
    }

    return NPERR_NO_ERROR;
}

NPError
NPP_NewStream(NPP instance, NPMIMEType type, NPStream *stream,
	      NPBool seekable, uint16 *stype)
{
    PluginInstance *This;

    if (instance == NULL)
	return NPERR_INVALID_INSTANCE_ERROR;

    This = (PluginInstance *) instance->pdata;

    if (This->opmode == NP_ASFILE || This->opmode == NP_ASFILEONLY)
	*stype = This->opmode;
    if (*stype != NP_ASFILEONLY) {
	This->pushed = 1;
	XtVaSetValues(This->buttRewind, XmNsensitive, False, NULL);
	XtVaSetValues(This->buttLoop, XmNsensitive, False,
		                      XmNset, False,
		                      NULL);
	This->mpegStream
	    = mpNewStreamPushed(This->display, XtWindowOfObject(This->video),
				findTopLevelWindow(This->display,
					       XtWindowOfObject(This->form)),
				This->dither,
				This->frameRate,
				1.0, 1.0,
				This->useSharedMemory, This->useOwnColormap);
	if (This->autoStart)
	    mpPlay(This->mpegStream);
    }
    return NPERR_NO_ERROR;
}

/* PLUGIN DEVELOPERS: These next 2 functions are directly relevant in
 * a plug-in which handles the data in a streaming manner. If you want
 * zero bytes because no buffer space is YET available, return 0. As
 * long as the stream has not been written to the plugin, Navigator
 * will continue trying to send bytes. If the plugin doesn't want
 * them, just return some large number from NPP_WriteReady(), and
 * ignore them in NPP_Write(). For a NP_ASFILE stream, they are still
 * called but can safely be ignored using this strategy. */

int32
NPP_WriteReady(NPP instance, NPStream *stream)
{
    PluginInstance* This;
    int accepting;

    if (instance == NULL)
	return 0;

    This = (PluginInstance*) instance->pdata;
    mpQueryWantedBytes(This->mpegStream, &accepting);
    return accepting;
}

int32
NPP_Write(NPP instance, NPStream *stream, int32 offset,
	  int32 len, void *buffer)
{
    PluginInstance *This;
    int n;

    if (instance == NULL)
	return -1;

    This = (PluginInstance *) instance->pdata;
    n = len;
    mpSendBytes(This->mpegStream, buffer, &n);
    return n;
}

NPError
NPP_DestroyStream(NPP instance, NPStream *stream, NPError reason)
{
    PluginInstance *This;

    if (instance == NULL)
	return NPERR_INVALID_INSTANCE_ERROR;
    This = (PluginInstance *) instance->pdata;
    if (This->pushed) {
	mpEndOfStream(This->mpegStream);
	XtVaSetValues(This->buttPlay, XmNsensitive, False, NULL);
	XtVaSetValues(This->buttPause, XmNsensitive, False, NULL);
	XtVaSetValues(This->buttStep, XmNsensitive, False, NULL);
    }
    return NPERR_NO_ERROR;
}

void
NPP_StreamAsFile(NPP instance, NPStream *stream, const char *fname)
{
    PluginInstance* This;

    if (instance != NULL)
	This = (PluginInstance*) instance->pdata;
    else
	return;

    This->pushed = 0;
    /* don't know why I need strdup. the filename is garbled in the
     * child process if strdup is not used. */
    This->mpegStream
	= mpNewStreamFromFile(strdup(fname),
			      This->display, XtWindowOfObject(This->video),
			      findTopLevelWindow(This->display,
						 XtWindowOfObject(This->form)),
			      This->dither,
			      This->frameRate, This->doLoop,
			      1.0, 1.0,
			      This->useSharedMemory, This->useOwnColormap);
    if (This->autoStart)
	mpPlay(This->mpegStream);
}

void
NPP_Print(NPP instance, NPPrint *printInfo)
{
    return;
}
